/******************************************************************************
**
** FILE NAME    : ifxmips_pmu_vr9.h
** PROJECT      : IFX UEIP
** MODULES      : PMU
**
** DATE         : 02 Sep 2009
** AUTHOR       : Lei Chuanhua
** DESCRIPTION  : IFX PMU vr9 specific header file
** COPYRIGHT    :       Copyright (c) 2009
**                      Infineon Technologies AG
**                      Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date        $Author         $Comment
** 02 Sep 2009  Lei Chuanhua    The first UEIP release
*******************************************************************************/
/*!
  \file ifxmips_pmu_vr9.h
  \ingroup IFX_PMU
  \brief PMU driver VRX specific file
*/
#ifndef IFXMIPS_PMU_VR9_H
#define IFXMIPS_PMU_VR9_H

#define IFX_PMU_MODULE_ID_IDX_MIN  IFX_PMU_MODULE_USB0_PHY
#define IFX_PMU_MODULE_ID_IDX_MAX  IFX_PMU_MODULE_DDR_CKE
extern ifx_pmu_t g_ifx_pmu ;

#ifdef CONFIG_IFX_PMU_POWER_GATING
static ifx_pmu_pg_priv_t g_ifx_pmu_pg[] = {
    {
        .pg = {IFX_POWER_DOMAIN(USB), 0},
        .real_domain = 0,
        .domain_name = "USB",
        .refcnt      = 0,
        .used        = 1,
    },
    {
        .pg = {IFX_POWER_DOMAIN(PCIE), 0},
        .real_domain = 1,
        .domain_name = "PCIE",
        .refcnt      = 0,
        .used        = 0,
    },
    {
        .pg = {IFX_POWER_DOMAIN(SLIC_TDM), 0},
        .real_domain = 2,
        .domain_name = "SLIC_TDM",
        .refcnt      = 0,
        .used        = 1,
    },
    {
        .pg = {IFX_POWER_DOMAIN(DEU), 0},
        .real_domain = 3,
        .domain_name = "DEU",
        .refcnt      = 0,
        .used        = 0,
    },
    {
        .pg = {IFX_POWER_DOMAIN(FPI_TOP), 0},
        .real_domain = 4,
        .domain_name = "FPI_TOP",
        .refcnt      = 0,
        .used        = 0,
    },
    {    
        .pg = {IFX_POWER_DOMAIN(PPE), 0},
        .real_domain = 5,
        .domain_name = "PPE",
        .refcnt      = 0,
        .used        = 1,
    },
    {
        .pg = {IFX_POWER_DOMAIN(SWITCH), 0},
        .real_domain = 8,
        .domain_name = "SWITCH",
        .refcnt      = 0,
        .used        = 0,
    },
    {
        .pg = {IFX_POWER_DOMAIN(MIPS), 0},
        .real_domain = 9,
        .domain_name = "MIPS",
        .refcnt      = 0,
        .used        = 1,
    },
    {
        .pg = {IFX_POWER_DOMAIN(DSL_DFE), 0},
        .real_domain = 11,
        .domain_name = "DSL_DFE",
        .refcnt      = 0,
        .used        = 1,
    },
};

static inline int ifx_pmu_power_domain_valid(u32 domain_idx)
{
    if (domain_idx == IFX_POWER_DOMAIN(USB) || domain_idx == IFX_POWER_DOMAIN(PCIE) ||
        domain_idx == IFX_POWER_DOMAIN(SLIC_TDM) || domain_idx == IFX_POWER_DOMAIN(DEU) ||
        domain_idx == IFX_POWER_DOMAIN(FPI_TOP) || domain_idx == IFX_POWER_DOMAIN(PPE) ||
        domain_idx == IFX_POWER_DOMAIN(SWITCH) || domain_idx == IFX_POWER_DOMAIN(MIPS) ||
        domain_idx == IFX_POWER_DOMAIN(DSL_DFE)) {
        if (g_ifx_pmu_pg[domain_idx].used) {
            return 1;
        }
        else {
            return 0;
        }
    }
    else {
        return 0;
    }
}
#endif /* CONFIG_IFX_PMU_POWER_GATING */
#endif /* IFXMIPS_PMU_VR9_H */

