/*------------------------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_AVM_WATCHDOG)
/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/fcntl.h>
#include <asm/ioctl.h>
#include <asm/errno.h>
#include <asm/uaccess.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
#include <asm/mips-boards/prom.h>
#else
#include <asm/prom.h>
#endif
#include <linux/timer.h>
#include <linux/ar7wdt.h>
#include "avm_sammel.h"
#include <asm/mach-ar7240/ar7240.h>

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
/*--- #define AVM_WATCHDOG_DEBUG ---*/

#if defined(AVM_WATCHDOG_DEBUG)
#define DBG_ERR(...)   printk(KERN_ERR __VA_ARGS__)
#define DBG_INFO(...)  printk(KERN_INFO __VA_ARGS__)
#else /*--- #if defined(AVM_WATCHDOG_DEBUG) ---*/
#define DBG_ERR(...)  
#define DBG_INFO(...)  
#endif /*--- #else ---*/ /*--- #if defined(AVM_WATCHDOG_DEBUG) ---*/

extern uint32_t ar7240_ahb_freq;

#define TIME_OUT_SECS   10

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void ar7wdt_hw_init(void) {
    DBG_ERR( "[ar7242:watchdog] start ...\n");
	ar7240_reg_wr(AR7240_WATCHDOG_TMR, (TIME_OUT_SECS * ar7240_ahb_freq));
    ar7240_reg_wr(AR7240_WATCHDOG_TMR_CONTROL, AR7240_WD_ACT_RESET);
    DBG_ERR( "[ar7242:watchdog] action: %u (hardware reset) - ticks: %u (= %u seconds * %u hz)\n", 
            AR7240_WD_ACT_RESET, 
            TIME_OUT_SECS * ar7240_ahb_freq, 
            TIME_OUT_SECS, ar7240_ahb_freq);
    return;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void ar7wdt_hw_deinit(void) {
    DBG_ERR( "[ar7242:watchdog] stop ...\n");
    ar7240_reg_wr(AR7240_WATCHDOG_TMR_CONTROL, AR7240_WD_ACT_NONE);
    return;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar7wdt_hw_reboot(void) {
    DBG_ERR("ar7wdt_hw_reboot!!\n");
    panic("ar7wdt_hw_reboot: watchdog expired\n");
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar7wdt_hw_trigger(void) {
#ifdef AVM_WATCHDOG_DEBUG
    unsigned int ticks_left = ar7240_reg_rd(AR7240_WATCHDOG_TMR);
    DBG_ERR( "[ar7242:watchdog] triggered %u.%u secs before reset\n", 
            ticks_left / ar7240_ahb_freq, (((ticks_left%ar7240_ahb_freq)*10U) / ar7240_ahb_freq) % 10);
#endif

	ar7240_reg_wr(AR7240_WATCHDOG_TMR, (TIME_OUT_SECS * ar7240_ahb_freq));
}

#endif /*--- #if defined(CONFIG_AVM_WATCHDOG) ---*/


