/*------------------------------------------------------------------------------------------*\
 *   Copyright (C) 2008,2009,...,2011 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
\*------------------------------------------------------------------------------------------*/

#include <linux/interrupt.h>
#include <linux/timer.h>
#include <linux/skbuff.h>
#include <linux/sched.h>
#include <linux/netdevice.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <linux/ioctl.h>
#include <linux/skbuff.h>
#include <asm/atomic.h>
#include <asm/mach_avm.h>
#if defined(CONFIG_AVM_POWER)
#include <linux/avm_power.h>
#endif /*--- #if defined(CONFIG_AVM_POWER) ---*/

#if !defined(CONFIG_NETCHIP_AR8216)
#define CONFIG_NETCHIP_AR8216
#endif
#if defined(CONFIG_MIPS_UR8)
#include <asm/mach-ur8/hw_mdio.h>
#endif /*--- #if defined(CONFIG_MIPS_UR8) ---*/
#include <linux/avm_event.h>
#include <linux/avm_cpmac.h>
#include <linux/ar_reg.h>
#include <linux/adm_reg.h>

#include "cpmac_if.h"
#include "cpmac_const.h"
#include "cpmac_debug.h"
#include "cpphy_const.h"
#include "cpphy_types.h"
#include "cpphy_mdio.h"
#include "cpphy_adm6996.h"
#include "cpphy_ar8216.h"
#include "cpphy_mgmt.h"
#include "cpmac_fusiv_if.h"
#include "cpmac_eth.h"

/*------------------------------------------------------------------------------------------*\
 * The configuration is defined in cpphy_adm6996.c                                          *
\*------------------------------------------------------------------------------------------*/
extern cpmac_switch_configuration_t switch_configuration[CPMAC_SWITCH_CONF_MAX_PRODUCTS][CPMAC_MODE_MAX_NO];
extern spinlock_t ar8216_lock;

/*------------------------------------------------------------------------------------------*\
 * die MDIO-Register des AR8216 sind 32bit gross, MDIO kann nur 16bit Zugriffe, d.h.
 * es muessen 2 Zugriffe stattfinden
 * zuerst wird die Registerpage ausgewaehlt, da die Registeradressen des AR8216 groesser als
 * der MDIO-Adressraum ist
\*------------------------------------------------------------------------------------------*/
unsigned int ar8216_mdio_read32(cpphy_mdio_t *mdio, unsigned int address) {
    unsigned int tmp, tmp1;
    unsigned int reg_addr;

    reg_addr = (address & 0xfffffffc);
    mdio_write(mdio, 0x18, 0, (reg_addr >> 9) & 0x3ff);
    tmp = mdio_read(mdio, 0x10 | ((reg_addr >> 6) & 0x7), (reg_addr >> 1) & 0x1F);
    /*--- DEB_TRC("[%s]: low16bit 0x%x\n", __FUNCTION__, tmp); ---*/
    tmp1 = mdio_read(mdio, 0x10 | ((reg_addr >> 6) & 0x7), ((reg_addr >> 1) & 0x1F) + 1);
    /*--- DEB_TRC("[%s]: high16bit 0x%x\n", __FUNCTION__, tmp1); ---*/
    return (tmp | (tmp1 << 16));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned short ar8216_mdio_read16(cpphy_mdio_t *mdio, unsigned short regadr) {
    unsigned short register_address, phy_address;

    register_address = (regadr & 0xFFFFFFFC) >> 1;
    phy_address = 0x10 | ((register_address >> 5) & 0x7);

    return mdio_read(mdio, phy_address, register_address & 0x1F);
}


/*------------------------------------------------------------------------------------------*\
 * zuerst die oberen 16bit schreiben
\*------------------------------------------------------------------------------------------*/
int ar8216_mdio_write32(cpphy_mdio_t *mdio, unsigned int address, unsigned int data) {
    unsigned int reg_addr;

    if(!(mdio->Mode & CPPHY_SWITCH_MODE_WRITE)) {
        return -1;
    }

    reg_addr = (address & 0xfffffffc);
    mdio_write(mdio, 0x18, 0, (reg_addr >> 9) & 0x3ff);
    mdio_write(mdio, 0x10 | ((reg_addr >> 6) & 0x7), ((reg_addr >> 1) & 0x1F) + 1, (unsigned short) (data >> 16));
    mdio_write(mdio, 0x10 | ((reg_addr >> 6) & 0x7), (reg_addr >> 1) & 0x1F, data & 0xFFFF);

    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8216_mdio_write16(cpphy_mdio_t *mdio, unsigned short regadr, unsigned short data) {
    unsigned short register_address, phy_address;

    register_address = (regadr & 0xFFFFFFFC) >> 1;
    phy_address = 0x10 | ((register_address >> 5) & 0x7);

    mdio_write(mdio, phy_address, register_address & 0x1F, data & 0xFFFF);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8216_dump_counters(cpphy_mdio_t *mdio) {
    char *cnt8216names[] = {
                            "RxBroad",
                            "RxPause",
                            "RxMulti",
                            "RxFcsErr",
                            "RxAlignErr",
                            "RxRunt",
                            "RxFragment",
                            "Rx64Byte",
                            "Rx128Byte",
                            "Rx256Byte",
                            "Rx512Byte",
                            "Rx1024Byte",
                            "RxMaxByte",
                            "RxTooLong",
                            "RxGoodByteLow",
                            "RxGoodByteHigh",
                            "RxBadByteLow",
                            "RxBadByteHigh",
                            "RxOverflow",
                            "Filtered",
                            "TxBroad",
                            "TxPause",
                            "TxMulti",
                            "TxUnderRun",
                            "Tx64Byte",
                            "Tx128Byte",
                            "Tx256Byte",
                            "Tx512Byte",
                            "Tx1024Byte",
                            "TxMaxByte",
                            "TxOverSize",
                            "TxByteLow",
                            "TxByteHigh",
                            "TxCollision",
                            "TxAbortCol",
                            "TxMultiCol",
                            "TxSingleCol",
                            "TxExcDefer",
                            "TxDefer",
                            "TxLateCol"
                           };
    unsigned int i;

    DEB_SUPPORT("Counters\n");
    DEB_SUPPORT("                    CPU port      Port 1      Port 2      Port 3      Port 4      Port 5\n");
    for(i = 0; i <= 0x9c; i += 4) {
        DEB_SUPPORT("  %14s  %10.u  %10.u  %10.u  %10.u  %10.u  %10.u\n", 
                    cnt8216names[i >> 2],
                    ar8216_mdio_read32(mdio, 0x19000 + i),
                    ar8216_mdio_read32(mdio, 0x190A0 + i),
                    ar8216_mdio_read32(mdio, 0x19140 + i),
                    ar8216_mdio_read32(mdio, 0x191E0 + i),
                    ar8216_mdio_read32(mdio, 0x19280 + i),
                    ar8216_mdio_read32(mdio, 0x19320 + i));
    }
    DEB_SUPPORT("\n");
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8316_dump_counters(cpphy_mdio_t *mdio) {
    char *cnt8316names[] = {
                            "RxBroad",
                            "RxPause",
                            "RxMulti",
                            "RxFcsErr",
                            "RxAlignErr",
                            "RxRunt",
                            "RxFragment",
                            "Rx64Byte",
                            "Rx128Byte",
                            "Rx256Byte",
                            "Rx512Byte",
                            "Rx1024Byte",
                            "Rx1518Byte",
                            "RxMaxByte",
                            "RxTooLong",
                            "RxGoodByteHigh",
                            "RxGoodByteLow",
                            "Reserved",
                            "Reserved",
                            "RxOverflow",
                            "Filtered",
                            "TxBroad",
                            "TxPause",
                            "TxMulti",
                            "TxUnderRun",
                            "Tx64Byte",
                            "Tx128Byte",
                            "Tx256Byte",
                            "Tx512Byte",
                            "Tx1024Byte",
                            "Tx1518Byte",
                            "TxMaxByte",
                            "TxOverSize",
                            "TxByteHigh",
                            "TxByteLow",
                            "TxCollision",
                            "TxAbortCol",
                            "TxMultiCol",
                            "TxSingleCol",
                            "TxExcDefer",
                            "TxDefer",
                            "TxLateCol"
                           };
    unsigned int i;

    DEB_SUPPORT("Counters\n");
    DEB_SUPPORT("                    CPU port      Port 1      Port 2      Port 3      Port 4      Port 5\n");
    for(i = 0; i <= 0xa4; i += 4) {
        DEB_SUPPORT("  %14s  %10.u  %10.u  %10.u  %10.u  %10.u  %10.u\n", 
                    cnt8316names[i >> 2],
                    ar8216_mdio_read32(mdio, 0x20000 + i),
                    ar8216_mdio_read32(mdio, 0x20100 + i),
                    ar8216_mdio_read32(mdio, 0x20200 + i),
                    ar8216_mdio_read32(mdio, 0x20300 + i),
                    ar8216_mdio_read32(mdio, 0x20400 + i),
                    ar8216_mdio_read32(mdio, 0x20500 + i));
    }
    DEB_SUPPORT("\n");
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void display_ath_regs(cpphy_mdio_t *mdio, enum ath_display_regs control) {
    int regs[] = {0, 0x8, 0x10, 0x14, 0x20, 0x24, 0x2c, 0x30, 0x34, 0x38, 0x3c, 0x40, 0x44, 
                  0x50, 0x54, 0x58, 0x5C, 0x60, 0x64, 0x68, 0x6C, 0x70, 0x74, 0x78, 0x80, 
                  0x94, 0x98, 0xb0, 0xb4, 0xb8, 0xbc, -1}; 
    int i = 0;
    unsigned short port, reg, control_reg;
#   define LENGTH 256
    char *string;

    string = kmalloc(LENGTH, GFP_KERNEL);
    if(string == NULL) {
        DEB_WARN("[%s] Out of memory!\n", __FUNCTION__);
        return;
    }
    switch(control) {
        case ATH_PORT_CONTROL:
            DEB_SUPPORT("Port control registers (all values in hex)\n");
            DEB_SUPPORT("          CPU port    Port 1    Port 2    Port 3    Port 4    Port 5\n");
            DEB_SUPPORT("             0x100     0x200     0x300     0x400     0x500     0x600\n");
            for(control_reg = 0; control_reg < 0x20; control_reg += 4) {
                unsigned int length = 0;

                length += snprintf(string, LENGTH, "  (0x%02x)  ", control_reg);

                for(port = 0x100; port < 0x700; port += 0x100) {
                    length += snprintf(string + length, LENGTH - length, "%8.x  ", ar8216_mdio_read32(mdio, port + control_reg));
                }
                DEB_SUPPORT("%s\n", string);
            }
            DEB_SUPPORT("\n");
            break;
        case ATH_CONTROL:
            DEB_SUPPORT("Global control registers (all values in hex)\n");
            while (regs[i] != -1) {
                DEB_SUPPORT("  0x%2.x  %8.x\n", regs[i], ar8216_mdio_read32(mdio, regs[i]));
                i++;
            }
            DEB_SUPPORT("\n");
            break;
        case ATH_MII:
            DEB_SUPPORT("MII registers (all values in hex)\n");
            DEB_SUPPORT("                      Port 1    Port 2    Port 3    Port 4    Port 5\n");
            for(reg = 0; reg <= 0x1D; reg++) {
                if(((reg > 0x19) && (reg < 0x1C)) || (reg == 0x17))
                    continue;
                DEB_SUPPORT("  (0x%02x)            %8.x  %8.x  %8.x  %8.x  %8.x\n", 
                        reg,
                        mdio_read(mdio, 0, reg),
                        mdio_read(mdio, 1, reg),
                        mdio_read(mdio, 2, reg),
                        mdio_read(mdio, 3, reg),
                        mdio_read(mdio, 4, reg)
                        );
            }
            DEB_SUPPORT("\n");
            DEB_SUPPORT("MDIO Debug registers\n");
            DEB_SUPPORT("                      Port 1    Port 2    Port 3    Port 4    Port 5\n");
            for(reg = 0; reg < 0x13; reg++) {
                if(reg == 0x11)
                    continue;
                for(port = 0; port < 5; port++) {
                    mdio_write(mdio, port, 0x1d, reg);
                }
                DEB_SUPPORT("  (0x%02x)            %8.x  %8.x  %8.x  %8.x  %8.x\n", 
                        reg,
                        mdio_read(mdio, 0, 0x1e),
                        mdio_read(mdio, 1, 0x1e),
                        mdio_read(mdio, 2, 0x1e),
                        mdio_read(mdio, 3, 0x1e),
                        mdio_read(mdio, 4, 0x1e)
                        );
                if(reg == 3)
                    reg = 0x10 - 1;
            }
            DEB_SUPPORT("\n");
            break;
    }
    kfree(string);
#   undef LENGTH
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8216_read_address_resolution_table(cpphy_mdio_t *mdio) {
#   if CPMAC_USE_DEBUG_LEVEL_SUPPORT
    int this;
    int count = -1;
    int status, status1;

    status = ar8216_mdio_read16(mdio, 0x15);
    DEB_TRC("[%s] rxerrors %d\n", __FUNCTION__, status);

    for(  ; ; ) {
        count++;
        do {
            status = ar8216_mdio_read32(mdio, AR8216_GLOBAL_AR_1) & AR_1_AT_BUSY;
        } while(status);
        ar8216_mdio_write16(mdio, 
                            AR8216_GLOBAL_AR_1,
                            0 | AR_1_AT_BUSY 
                              | AR_1_AT_FUNC_VALUE(6)
                              | AR_1_AT_PORT_NUM_VALUE(0)
                           );

        status1 = ar8216_mdio_read32(mdio, AR8216_GLOBAL_AR_1);
        this = ar8216_mdio_read32(mdio, AR8216_GLOBAL_AR_2);
        status = ar8216_mdio_read32(mdio, AR8216_GLOBAL_AR_3);

        if(this == 0) {
            DEB_TRC("[%s] index %d: empty\n", __FUNCTION__, count);
            break;
        }

        DEB_SUPPORT("[%s] index %d: %02x:%02x:",
                    __FUNCTION__,
                    count,
                    (this >> 24) & 0xFF,
                    (this >> 16) & 0xFF);
        DEB_SUPPORT("[%s] %02x:%02x:%02x:%02x ",
                    __FUNCTION__,
                    (this >> 8) & 0xFF,
                    (this >> 0) & 0xFF,
                    (status1 >> AR_1_AT_ADDR_BYTE4_SHIFT) & 0xFF,
                    (status1 >> AR_1_AT_ADDR_BYTE5_SHIFT) & 0xFF);
        DEB_SUPPORT("[%s] %s%s%s%s%s \n\tStatus: %s DES 0x%x\n" , 
                    __FUNCTION__,
                    status & AR_3_AT_PRIORITY_EN ? " AT_PRIORITY_EN" : "",
                    status & AR_3_AT_MIRROR_EN ? " AT_MIRROR_EN" : "",
                    status & AR_3_SA_DROP_EN ? " SA_DROP_EN" : "",
                    status & AR_3_REDIRECT_TO_CPU ? " REDIRECT_TO_CPU" : "",
                    status & AR_3_COPY_TO_CPU  ? " COPY_TO_CPU " : "",
                    (status & AR_3_AT_STATUS_MASK) == AR_3_AT_STATUS_VALUE(0) ? "entry is empty" :
                    (status & AR_3_AT_STATUS_MASK) == AR_3_AT_STATUS_VALUE(1) ? "entry is static und valid" :
                    (status & AR_3_AT_STATUS_MASK) == AR_3_AT_STATUS_VALUE(2) ? "entry is valid and has been accessed since the last aging process" :
                    (status & AR_3_AT_STATUS_MASK) == AR_3_AT_STATUS_VALUE(3) ? "entry is valid and has not been accessed since the last aging process" : "unknown",
                    (status & AR_3_DES_PORT_MASK) >> AR_3_DES_PORT_SHIFT);
    } 
    DEB_SUPPORT("\n");
#   endif /*--- #if CPMAC_USE_DEBUG_LEVEL_SUPPORT ---*/
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void ar8216_wait_VLAN_table(cpphy_mdio_t *mdio) {
    unsigned int status;
    do {
        schedule();
        status = ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_1);
    } while(status & VLAN_TABLE_1_VT_BUSY);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8316_read_VLAN_table(cpphy_mdio_t *mdio) {
    unsigned short vid = 0;
    unsigned int RegData;

    ar8216_wait_VLAN_table(mdio);
    DEB_SUPPORT("VLAN table\n");
    DEB_SUPPORT("  VID           port map\n");
    for( ;; ) {
        unsigned char valid;
        unsigned short members;

        ar8216_mdio_write32(mdio, 
                            AR8216_GLOBAL_VLAN_TABLE_1,
                              VLAN_TABLE_1_VT_FUNC_VALUE(VLAN_TABLE_FUNC_READ_NEXT_ENTRY)
                            | VLAN_TABLE_1_VT_BUSY
                            /*--- | VLAN_TABLE_1_VT_PORT_NUM_VALUE(0) ---*/
                            | VLAN_TABLE_1_VID_VALUE(vid)
                            /*--- | VLAN_TABLE_1_VT_PRI_VALUE(0) ---*/
                            /*--- | VLAN_TABLE_1_VT_PRI_EN ---*/
                           );
        ar8216_wait_VLAN_table(mdio);
        RegData = ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_1);
        vid = (RegData & VLAN_TABLE_1_VID_MASK) >> VLAN_TABLE_1_VID_SHIFT;
        if(vid == 0) {
            break; /* Done */
        }
        RegData = ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_2);
        valid = (RegData & VLAN_TABLE_2_VT_VALID) ? 1 : 0;
        members = (RegData & VLAN_TABLE_2_VID_MEM_MASK) >> VLAN_TABLE_2_VID_MEM_SHIFT;
        DEB_SUPPORT("  %04u (%#03x)  ports %#02x %s\n", vid, vid, members, valid ? "" : "not valid!");
    }
    DEB_SUPPORT("\n");
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void switch_dump_atheros(cpphy_mdio_t *mdio) {
    display_ath_regs(mdio, ATH_PORT_CONTROL);
    display_ath_regs(mdio, ATH_CONTROL);
    display_ath_regs(mdio, ATH_MII);
    if(mdio->f->dump_counters) {
        mdio->f->dump_counters(mdio);
    }
    ar8316_read_VLAN_table(mdio);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8216_softreset(cpphy_mdio_t *mdio) {
    unsigned int tmp = (1<<31);

    DEB_TRC("[%s] softreset switch\n", __FUNCTION__);
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_DEVICE_ID, tmp);
    mdelay(200);

    while (tmp & (1<<31)) {
        tmp = ar8216_mdio_read32(mdio, AR8216_GLOBAL_DEVICE_ID);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8216_reset_vlans(cpphy_mdio_t *mdio) {
    unsigned int status;

    do {
        status = ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_1);
        if(status & VLAN_TABLE_1_VT_BUSY) {
            printk(KERN_ERR "VLAN_TABLE_1_VT_BUSY !\n");
            schedule();
        }
    } while(status & VLAN_TABLE_1_VT_BUSY);

    ar8216_mdio_write32(mdio, AR8216_GLOBAL_VLAN_TABLE_1, 0 /*--- 0x0001000a); ---*/
            | VLAN_TABLE_1_VT_FUNC_VALUE(0x1)  /*--- flush all entries ---*/
            | VLAN_TABLE_1_VT_BUSY
            /*--- | VLAN_TABLE_1_VT_FULL_VIO ---*/
            /*--- | VLAN_TABLE_1_VT_PORT_NUM_VALUE(0) ---*/
            /*--- | VLAN_TABLE_1_VID_VALUE(vid) ---*/
            /*--- | VLAN_TABLE_1_VT_PRI_VALUE(0) ---*/
            /*--- | VLAN_TABLE_1_VT_PRI_EN ---*/
            );
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void add_vlan_vid(cpphy_mdio_t *mdio, unsigned int vid, unsigned int mem_mask) {
    unsigned int status;

    do {
        /* TODO Separate between AR8216 and AR8316 */
        status = ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_1);
        /*--- if(status & VLAN_TABLE_1_VT_FULL_VIO) { ---*/
            /*--- printk(KERN_ERR "VLAN_TABLE_1_VT_FULL_VIO !\n"); ---*/
            /*--- return; ---*/
        /*--- } ---*/
        if(status & VLAN_TABLE_1_VT_BUSY) {
            printk(KERN_ERR "VLAN_TABLE_1_VT_BUSY !\n");
            schedule();
        }
    } while(status & VLAN_TABLE_1_VT_BUSY);

    ar8216_mdio_write32(mdio, AR8216_GLOBAL_VLAN_TABLE_2, 0 /*--- 0x0000081f); ---*/   
            | VLAN_TABLE_2_VID_MEM_VALUE(mem_mask)
            | VLAN_TABLE_2_VT_VALID           
            );

    ar8216_mdio_write32(mdio, AR8216_GLOBAL_VLAN_TABLE_1, 0 /*--- 0x0001000a); ---*/
            | VLAN_TABLE_1_VT_FUNC_VALUE(0x2)  /*--- load entry ---*/
            | VLAN_TABLE_1_VT_BUSY
            /*--- | VLAN_TABLE_1_VT_FULL_VIO ---*/
            /*--- | VLAN_TABLE_1_VT_PORT_NUM_VALUE(0) ---*/
            | VLAN_TABLE_1_VID_VALUE(vid)
            /*--- | VLAN_TABLE_1_VT_PRI_VALUE(0) ---*/
            /*--- | VLAN_TABLE_1_VT_PRI_EN ---*/
            );
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void ar_generic_init(cpphy_mdio_t *mdio) {
    unsigned char port;
    unsigned int tmp;

    DEB_INFOTRC("[%s]\n", __FUNCTION__);

	ar8216_ar_init();

    spin_lock_init(&ar8216_lock);

    mdio->run_ar_queue_without_removal_counter = 0;

    /*--- for(port = 0; port < 5; port++) { ---*/
    for(port = 0; port < 6; port++) {
        mdio_write(mdio, 
                   port, 
                   AR8216_MII_AUTO_NEG_ADV, 
                   ATHR_ADVERTISE_ALL);
        mdio_write(mdio, 
                   port, 
                   AR8216_MII_CONTROL, 
                   AR8216_PHYCTRL_SW_RESET | AR8216_PHYCTRL_AUTONEG_EN);
    }

    mdelay(200);
    /*--- for(port = 1; port < 5; port++) { ---*/
    for(port = 1; port < 6; port++) {
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_STATUS_OFFSET, 
                            ATH_PORT_STATUS_LINKEN);
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_CONTROL_OFFSET,
                            ATH_PORT_CTRL_LEARN_EN
                            | ATH_PORT_CTRL_EG_VLAN_MODE(1)
                            | ATH_PORT_CTRL_PORTSTATE_FORWARD
                            );
        /* Default VID 1, port member only CPU port */
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_VLAN_OFFSET, 
                              ATH_PORT_VLAN_VID_VALUE(1)
                            | ATH_PORT_VLAN_VID_MEM_VALUE(1)
                           );
    }

    ar8216_mdio_write32(mdio, 0x104, 0x6004);

    /* Disable unused ports as far as possible */
    for(port = 0; port < mdio->switch_config.ports; port++) {
        if(mdio->switch_config.used_portset & (1 << port)) {
            continue;
        }
        DEB_INFOTRC("[%s] Disable unused port %u\n", __FUNCTION__, port);
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_STATUS_OFFSET, 
                            0);
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_CONTROL_OFFSET, 
                            ATH_PORT_CTRL_PORTSTATE_DISABLE);
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_VLAN_OFFSET, 
                            ATH_PORT_CTRL_PORTSTATE_DISABLE);
        mdio_write(mdio, port, 0x0, 0xc00);
    }

    /* Allow bigger packet size: CPPHY_MAX_RX_BUFFER_SIZE + 4 bytes for double tagged jumbo packets */
    tmp = ar8216_mdio_read32(mdio, AR8216_GLOBAL_CONTROL);
    tmp &= 0xfffffc00;
    tmp |= (CPPHY_MAX_RX_BUFFER_SIZE + 4);
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_CONTROL, tmp);

    /* Enable MIB counters */
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_MIB, 0x40000000);

    mdio->state = CPPHY_MDIO_ST_LINKED;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_ar8216_init(cpphy_mdio_t *mdio) {
    unsigned int tmp;

    ar8216_softreset(mdio);

    tmp = ar8216_mdio_read32(mdio, AR8216_GLOBAL_DEVICE_ID);
    tmp |= 0x500000;
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_DEVICE_ID, (1 << 24) | tmp);  /*--- change MII-Clockphase by 180 ---*/

    ar_generic_init(mdio);

    ar8216_mdio_write32(mdio, 0x38, 0xc000050e);      /*--- undefined lt. Datenblatt ---*/

    /*--- ar8216_mdio_write32(mdio, 0x60, 0xffffffff); ---*/
    ar8216_mdio_write32(mdio, 0x60, 0x0);
    ar8216_mdio_write32(mdio, 0x64, 0xaaaaaaaa);
    ar8216_mdio_write32(mdio, 0x68, 0x55555555);    
    /*--- ar8216_mdio_write32(mdio, 0x6c, 0x0); ---*/    
    ar8216_mdio_write32(mdio, 0x6c, 0xffffffff);    
    /*--- ar8216_mdio_write32(mdio, 0x70, 0x41af); ---*/
    ar8216_mdio_write32(mdio, 0x70, 0xfa41);

    /*--- ar8216_mdio_write32(mdio, 0x78, (1 << 8)); ---*/
    
    /* Enable address table entry aging (18 * 17 seconds) */
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_AR_CONTROL, AR_CONTROL_AGE_EN | AR_CONTROL_AGE_TIME(CPMAC_AR_AGE_TIME));

    /* Enable CPU port */
    ar8216_mdio_write32(mdio, 
                        AR8216_PORT0_CONTROL_BASIS,
                          ATH_PORT_STATUS_SPEED_100M
                        | ATH_PORT_STATUS_TXMACEN
                        | ATH_PORT_STATUS_RXMACEN
                        | ATH_PORT_STATUS_TXFLOWEN
                        | ATH_PORT_STATUS_RXFLOWEN
                        | ATH_PORT_STATUS_FULLDUPLEX
                        | ATH_PORT_STATUS_PHY_LINKUP
                        | ATH_PORT_STATUS_LINK_PAUSE
                        | ATH_PORT_STATUS_ASYNC_PAUSE
                       );
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_CPUPORT, 0xf0);

    if(mdio->f->ar_work_item) {
        cpphy_mgmt_work_add(mdio, CPMAC_WORK_UPDATE_MAC_TABLE, mdio->f->ar_work_item, CPMAC_ARL_UPDATE_INTERVAL);
    } else {
        DEB_ERR("[%s] No ar_work_item found!\n", __FUNCTION__);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_ar8226_init(cpphy_mdio_t *mdio) {
    ar8216_softreset(mdio);

    ar_generic_init(mdio);

    /* Enable CPU port */
    ar8216_mdio_write32(mdio, 
                        AR8216_PORT0_CONTROL_BASIS,
                        ATH_PORT_STATUS_SPEED_100M
                        | ATH_PORT_STATUS_TXMACEN
                        | ATH_PORT_STATUS_RXMACEN
                        | ATH_PORT_STATUS_TXFLOWEN
                        | ATH_PORT_STATUS_RXFLOWEN
                        | ATH_PORT_STATUS_FULLDUPLEX
                       );
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_CPUPORT, 0x1f0);

    ar8216_mdio_write32(mdio, 0x0,  0x201);
    ar8216_mdio_write32(mdio, 0x04, 0x80000400);
    ar8216_mdio_write32(mdio, 0x10, 0x80061B20);
    ar8216_mdio_write32(mdio, 0x2c, 0xfe7f007f);

    if(mdio->f->ar_work_item) {
        cpphy_mgmt_work_add(mdio, CPMAC_WORK_UPDATE_MAC_TABLE, mdio->f->ar_work_item, CPMAC_ARL_UPDATE_INTERVAL);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_ar8316_mac5_enable(cpphy_mdio_t *mdio, unsigned char enable) {
    unsigned int strap_on = ar8216_mdio_read32(mdio, AR8316_GLOBAL_PWR_ON_STRAP);
    unsigned int port_status = ar8216_mdio_read32(mdio, AR8216_PORT5_CONTROL_BASIS + AR8216_PORT_STATUS_OFFSET);
    if(enable) {
        strap_on |= AR8316_GLOBAL_PWR_ON_STRAP_PHY4_RGMII_EN;
        port_status |=   ATH_PORT_STATUS_SPEED_1000M
                       | ATH_PORT_STATUS_TX_HALF_FLOW_EN
                       | ATH_PORT_STATUS_PHY_LINKUP;
    } else {
        strap_on &= ~AR8316_GLOBAL_PWR_ON_STRAP_PHY4_RGMII_EN;
        port_status &= ~(  ATH_PORT_STATUS_SPEED_1000M
                         | ATH_PORT_STATUS_TX_HALF_FLOW_EN
                         | ATH_PORT_STATUS_PHY_LINKUP);
    }
    ar8216_mdio_write32(mdio, AR8316_GLOBAL_PWR_ON_STRAP, strap_on);
    ar8216_mdio_write32(mdio, AR8216_PORT5_CONTROL_BASIS + AR8216_PORT_STATUS_OFFSET, port_status);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_ar8316_init(cpphy_mdio_t *mdio) {
    unsigned int tmp;

    ar8216_softreset(mdio);

    ar_generic_init(mdio);

    tmp = ar8216_mdio_read32(mdio, 0x108);
    tmp |= ((1<<24)|(1<<2));
    ar8216_mdio_write32(mdio, 0x108, tmp);

    /* Enable CPU port */
    ar8216_mdio_write32(mdio, 
                        AR8216_PORT0_CONTROL_BASIS,
                        ATH_PORT_STATUS_SPEED_1000M
                        | ATH_PORT_STATUS_TXMACEN
                        | ATH_PORT_STATUS_RXMACEN
                        | ATH_PORT_STATUS_TXFLOWEN
                        | ATH_PORT_STATUS_RXFLOWEN
                        | ATH_PORT_STATUS_FULLDUPLEX
                       );
    ar8216_mdio_write32(mdio, AR8216_GLOBAL_CPUPORT, 0xf0);

    /*--- ar8216_mdio_write32(mdio, 0x2c, 0x003f003f); ---*/
    /* Define portsets for - broadcasts adresses              *
     *                     - unknown unicast adresses         *
     *                     - IGMP snooping of fast join/leave */
    ar8216_mdio_write32(mdio, 0x2c, 0x3f3f3f);
    ar8216_mdio_write32(mdio,
                        AR8316_GLOBAL_PWR_ON_STRAP,
                          AR8316_GLOBAL_PWR_ON_STRAP_RESERVED
                        | AR8316_GLOBAL_PWR_ON_STRAP_MAC0_GMII_EN
                        | AR8316_GLOBAL_PWR_ON_STRAP_MAC0_MAC_MODE
                        | AR8316_GLOBAL_PWR_ON_STRAP_MAC5_PHY_MODE
                        | AR8316_GLOBAL_PWR_ON_STRAP_LED_OPEN_EN);
    ar8216_mdio_write32(mdio, 
                        AR8216_PORT5_CONTROL_BASIS + AR8216_PORT_STATUS_OFFSET,  
                          ATH_PORT_STATUS_SPEED_10M
                        | ATH_PORT_STATUS_TXMACEN
                        | ATH_PORT_STATUS_RXMACEN
                        | ATH_PORT_STATUS_TXFLOWEN
                        | ATH_PORT_STATUS_RXFLOWEN
                        | ATH_PORT_STATUS_FULLDUPLEX
                       );

    /* Queue priorization and queue length limits */
    if(mdio->switch_predefined_configs == &(switch_configuration[CPMAC_SWITCH_CONF_MAGPIE][0])) {
        unsigned char port;

        for(port = 1; port <= 4; port++) {
            /* Limit all external port queue sizes to 128 */
            ar8216_mdio_write32(mdio, (port + 1) * 0x100 + 0x18, 0x02000000 + ((128 / 4) << 16));
            /* Reduce the priority of the external ports */
            ar8216_mdio_write32(mdio, (port + 1) * 0x100 + 0x10, 0x80019);
        }
    }

    if(mdio->f->ar_work_item) {
        cpphy_mgmt_work_add(mdio, CPMAC_WORK_UPDATE_MAC_TABLE, mdio->f->ar_work_item, CPMAC_ARL_UPDATE_INTERVAL);
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/

void ar_ar8216_deinit(cpphy_mdio_t *mdio __attribute__ ((unused)))
{
   ar8216_ar_deinit();
}

void ar_ar8226_deinit(cpphy_mdio_t *mdio __attribute__ ((unused)))
{
   ar8216_ar_deinit();
}

void ar_ar8316_deinit(cpphy_mdio_t *mdio __attribute__ ((unused)))
{
   ar8216_ar_deinit();
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar_switch_test_link(cpphy_mdio_t *mdio, unsigned char port) {
    unsigned short status = 0, link = 0;

    if(!cpmac_global.power.port_on[port]) {
        /* Without power, the link information would be wrong */
        link = 0;
    } else {
        /* Check real time link information */
        status = mdio_read(mdio, port, 0x11);
        link = (status & ATHR_PHY_STATUS_LINK) ? 1 : 0;
    }
    if(cpmac_global.event_data.port[port].link != link) {
        /* Link changed. Update link status */
        cpmac_global.event_data.port[port].speed100 = (((status & ATHR_PHY_STATUS_SPEED_MASK) >> ATHR_PHY_STATUS_SPEED_SHIFT) == 1) ? 1 : 0;
        cpmac_global.event_data.port[port].speed = avm_event_ethernet_speed_no_link;
        if(link) {
            switch((status & ATHR_PHY_STATUS_SPEED_MASK) >> ATHR_PHY_STATUS_SPEED_SHIFT) { 
                case 0: cpmac_global.event_data.port[port].speed = avm_event_ethernet_speed_10M; break;
                case 1: cpmac_global.event_data.port[port].speed = avm_event_ethernet_speed_100M; break;
                case 2: cpmac_global.event_data.port[port].speed = avm_event_ethernet_speed_1G; break;
                default: /* Nothing to change */ break;
            }
        }
        cpmac_global.event_data.port[port].fullduplex = (status & ATHR_PHY_STATUS_DUPLEX) ? 1 : 0;
        mdio->event_update_needed++;
    }
    cpmac_global.event_data.port[port].link = link;

    return link;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar_check_link_orig(cpphy_mdio_t *mdio) {
    unsigned int link_status = 0;
    unsigned char phy;

    for(phy = 0; phy < AR8216_MAX_PHYS; phy++) {
        link_status |= ar_switch_test_link(mdio, phy) << (phy + 1);
    }

    mdio->linked = link_status;

    return mdio->linked;
}

 
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar_check_link_emv(cpphy_mdio_t *mdio) {
    return mdio->linked;
}

 
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_switch_port_power_on_boot_orig(cpphy_mdio_t *mdio, unsigned short portset) {
    unsigned char port;

    DEB_WARN("[%s] This will take approx. 12 seconds!\n", __FUNCTION__);
    for(port = 0; port < 5; port++) {
        if((1 << port) & portset) {
            mdio_write(mdio, port, 0x0,  0x3100);
            mdio_write(mdio, port, 0x1d, 0x3);
            mdio_write(mdio, port, 0x1e, 0x11);
        }
    }
    /*--- ssleep(4); ---*/
    for(port = 0; port < 5; port++) {
        if((1 << port) & portset) {
            mdio_write(mdio, port, 0x1d, 0x2);
            mdio_write(mdio, port, 0x1e, 0x3200);
        }
    }
    /*--- ssleep(4); ---*/
    for(port = 0; port < 5; port++) {
        if((1 << port) & portset) {
            mdio_write(mdio, port, 0x1d, 0x1);
            mdio_write(mdio, port, 0x1e, 0x1d0);
        }
    }
    /*--- ssleep(4); ---*/
    for(port = 0; port < 5; port++) {
        if((1 << port) & portset) {
            mdio_write(mdio, port, 0x1d, 0x0);
            mdio_write(mdio, port, 0x1e, 0x024e);
        }
    }
    ssleep(4);
    for(port = 0; port < 5; port++) {
        if((1 << port) & portset) {
            mdio_write(mdio, port, 0x0, 0xffff);
        }
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar_switch_test_duplex(cpphy_mdio_t *mdio, unsigned char port) {
    unsigned short temp;

    /* Check real time duplex resolved information */
    temp = mdio_read(mdio, port, 0x11);
    return (temp & ATHR_PHY_STATUS_SPEED_DUPLEX);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8316_switch_port_power(cpphy_mdio_t *mdio, 
                              unsigned char port, 
                              unsigned char power_on) {
    unsigned short RegData;

    DEB_DEBUG("[%s] %lu Port %u %s\n", __FUNCTION__, jiffies, port, power_on ? "on" : "off"); 

    RegData = mdio_read(mdio, port, AR8216_MII_CONTROL);
    if(power_on) {
        RegData &= ~AR8216_PHYCTRL_POWERDOWN;
        RegData |= AR8216_PHYCTRL_AUTONEG_RESTART;
    } else {
        RegData |= AR8216_PHYCTRL_POWERDOWN;
    }
    mdio_write(mdio, port, AR8216_MII_CONTROL, RegData);
}


#if defined(POWERMANAGEMENT_THROTTLE_ETH)
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar8316_switch_port_speed_throttle(cpphy_mdio_t *mdio,
                                       unsigned char port,
                                       unsigned char do_throttle) {
    unsigned short RegData;

    do_throttle = do_throttle ? 1 : 0;

    cpphy_mgmt_work_schedule(mdio, CPMAC_WORK_TICK, 5 * HZ);
    if(do_throttle) {
        /* Disable 1000 Base T negotiation capability */
        DEB_INFO("[%s] Throttling port %u speeds to 100 MBit/s\n", __FUNCTION__, port);
        mdio_write(mdio, port, AR8316_MII_1000BASET_CTRL, 0);
    } else {
        /* Enable 1000 Base T negotiation capability */
        DEB_INFO("[%s] Unthrottling port %u speeds\n", __FUNCTION__, port);
        mdio_write(mdio, port, AR8316_MII_1000BASET_CTRL, 0x200);
    }

    /* Now reset the port to let the change take effect */
    RegData = mdio_read(mdio, port, AR8216_MII_CONTROL);
    mdio_write(mdio, port, AR8216_MII_CONTROL, RegData | AR8216_PHYCTRL_SW_RESET);
}
#endif /*--- #if defined(POWERMANAGEMENT_THROTTLE_ETH) ---*/


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_switch_port_power_orig(cpphy_mdio_t *mdio, 
                               unsigned char port, 
                               unsigned char power_on) {
    /*--- DEB_TEST("[%s] %lu Port %u %s\n", __FUNCTION__, jiffies, port, power_on ? "on" : "off"); ---*/ 
    if(power_on) {
        mdio_write(mdio, port, 0x1d, 0x0);
        mdio_write(mdio, port, 0x1e, 0x124e);
        mdio_write(mdio, port, 0x1d, 0x3);
        mdio_write(mdio, port, 0x1e, 0x11);
        mdio_write(mdio, port, 0x1d, 0x2);
        mdio_write(mdio, port, 0x1e, 0x3220);
        mdio_write(mdio, port, 0x1d, 0x1);
        mdio_write(mdio, port, 0x1e, 0x1d0);
        mdio_write(mdio, port, 0x1d, 0x0);
        mdio_write(mdio, port, 0x1e, 0x024e);
        mdio_write(mdio, port, 0x0,  0xffff);
    } else {
        mdio_write(mdio, port, 0x0,  0x1200);
        mdio_write(mdio, port, 0x1d, 0x0);
        mdio_write(mdio, port, 0x1e, 0x124e);
        mdio_write(mdio, port, 0x1d, 0x1);
        mdio_write(mdio, port, 0x1e, 0x10);
        mdio_write(mdio, port, 0x1d, 0x2);
        mdio_write(mdio, port, 0x1e, 0x3000);
        mdio_write(mdio, port, 0x1d, 0x3);
        mdio_write(mdio, port, 0x1e, 0x0000);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_boot_power_on_emv(cpphy_mdio_t *mdio) {
    unsigned char port;

    for(port = 0; port < 4; port++) {
        /* Switch on port again */
        DEB_INFOTRC("[%s] port %u: restart after CPPHY_POWER_MODE_OFF\n", __FUNCTION__, port);
        mdio_write(mdio, port, 0x1d, 0x0);
        mdio_write(mdio, port, 0x1e, 0x124e);
        mdio_write(mdio, port, 0x1d, 0x3);
        mdio_write(mdio, port, 0x1e, 0x11);
        mdio_write(mdio, port, 0x1d, 0x2);
        mdio_write(mdio, port, 0x1e, 0x3220);
        mdio_write(mdio, port, 0x1d, 0x1);
        mdio_write(mdio, port, 0x1e, 0x1d0);
        mdio_write(mdio, port, 0x1d, 0x0);
        mdio_write(mdio, port, 0x1e, 0x024e);
        mdio_write(mdio, port, 0x0,  0x3100);
    }
    ssleep(4);

    for(port = 0; port < 4; port++) {
        /* Force port into CPPHY_POWER_MODE_ON_MDI_MLT3 to prevent initial 10 MBit/s links */
        DEB_INFOTRC("[%s] port %u: reset as MDI/MLT3\n", __FUNCTION__, port);
        mdio_write(mdio, port, 0x14, 0x2c);
        mdio_write(mdio, port, 0x10, 0x800);
        mdio_write(mdio, port, 0x0,  0xa100);
        cpmac_global.power.port_mode[port] = CPPHY_POWER_MODE_ON_MDI_MLT3;
    }
    ssleep(2);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_switch_port_power_emv(cpphy_mdio_t *mdio, 
                              unsigned char port, 
                              cpphy_switch_powermodes_t mode) {
    cpmac_global.power.time_to_wait_to[port] = jiffies;

    if(mode == cpmac_global.power.port_mode[port]) {
        DEB_TEST("[%s] Mode %u already set for port %u\n", __FUNCTION__, mode, port);
        return;
    }

    switch(mode) {
        case CPPHY_POWER_MODE_OFF:
            /* TODO Check, whether this sequence works in all contexts (even in mute mode?) */
            DEB_INFOTRC("[%s] port %u: CPPHY_POWER_MODE_OFF\n", __FUNCTION__, port);
            mdio_write(mdio, port, 0x0,  0x1200);
            mdio_write(mdio, port, 0x1d, 0x0);
            mdio_write(mdio, port, 0x1e, 0x124e);
            mdio_write(mdio, port, 0x1d, 0x1);
            mdio_write(mdio, port, 0x1e, 0x10);
            mdio_write(mdio, port, 0x1d, 0x2);
            mdio_write(mdio, port, 0x1e, 0x3000);
            mdio_write(mdio, port, 0x1d, 0x3);
            mdio_write(mdio, port, 0x1e, 0x0000);
            break;

        case CPPHY_POWER_MODE_ON_IDLE_MDI:
            DEB_INFOTRC("[%s] port %u: CPPHY_POWER_MODE_ON_IDLE_MDI\n", __FUNCTION__, port);
            mdio_write(mdio, port, 0x10, 0x800);
            mdio_write(mdio, port, 0x00, 0xa100);
            break;

        case CPPHY_POWER_MODE_ON:
#           if CPMAC_USE_DEBUG_LEVEL_INFO
            {
                unsigned short temp = mdio_read(mdio, port, 0x11);
                DEB_INFO("[%s] port %u: CPPHY_POWER_MODE_ON %u MBit/s, %s duplex, MDI%s\n", __FUNCTION__, 
                         port,
                         (temp & ATHR_PHY_STATUS_SPEED_MASK) ? 100 : 10,
                         (temp & ATHR_PHY_STATUS_DUPLEX) ? "full" : "half",
                         (temp & ATHR_PHY_STATUS_MDI_CROSSOVER) ? "X" : ""
                        );
            }
#           endif /*--- #if CPMAC_USE_DEBUG_LEVEL_INFO ---*/
            /* If this is the first linked port, lock port 0 to MDIX */
            if(mdio->linked == 0) {
                if(port != 0) {
                    ar_switch_port_power_emv(mdio, 0, CPPHY_POWER_MODE_ON_MDIX);
                }
            }
            mdio->linked |= (1 << port);
            DEB_TEST("[%s] ports linked: %#x\n", __FUNCTION__, mdio->linked);
            if((port != 0) || (mdio->linked != (1 << 0))) {
                if(cpmac_global.power.port_mode[port] == CPPHY_POWER_MODE_ON_MDI_MLT3) {
                    mdio_write(mdio, port, 0x10, 0x860);
                    mdio_write(mdio, port, 0x0,  0xffff);
                }
            }
            break;

        case CPPHY_POWER_MODE_ON_MDI_MLT3:
            if(ar_switch_test_link(mdio, port)) {
                ar_switch_port_power_emv(mdio, port, CPPHY_POWER_MODE_ON);
                return;
            }
            DEB_INFOTRC("[%s] port %u: CPPHY_POWER_MODE_ON_MDI_MLT3\n", __FUNCTION__, port);
            mdio_write(mdio, port, 0x14, 0x2c);
            mdio_write(mdio, port, 0x10, 0x800);
            mdio_write(mdio, port, 0x0,  0xa100);
            break;

        case CPPHY_POWER_MODE_ON_MDIX:
            if(ar_switch_test_link(mdio, port)) {
                ar_switch_port_power_emv(mdio, port, CPPHY_POWER_MODE_ON);
                return;
            }
            DEB_INFOTRC("[%s] port %u: CPPHY_POWER_MODE_ON_MDIX\n", __FUNCTION__, port);
            mdio_write(mdio, port, 0x14, 0x2c);
            mdio_write(mdio, port, 0x10, 0x820);
            mdio_write(mdio, port, 0x0,  0xffff);
            break;

        case CPPHY_POWER_MODE_LINK_LOST:
            /* Do nothing */
            break;

        default:
            DEB_ERR("[%s] port %u: Illegal mode (%u)\n", __FUNCTION__, port, mode);
            return;
    }
    cpmac_global.power.port_mode[port] = mode;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_vlan_table_wait(cpphy_mdio_t *mdio) {
    while(VLAN_TABLE_1_VT_BUSY & ar8216_mdio_read32(mdio, AR8216_GLOBAL_VLAN_TABLE_1)) {
        DEB_TRC("[%s] VLAN table busy\n", __FUNCTION__);
        schedule();
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_vlan_clear(cpphy_mdio_t *mdio) {
    if(!(mdio->Mode & CPPHY_SWITCH_MODE_WRITE))
        return; /*--- CPMAC_ERR_NO_VLAN_POSSIBLE; ---*/

    ar_vlan_table_wait(mdio);
    ar8216_mdio_write32(mdio, 
                        AR8216_GLOBAL_VLAN_TABLE_1, 
                          VLAN_TABLE_1_VT_FUNC_VALUE(VLAN_TABLE_FUNC_FLUSH_ALL)
                        | VLAN_TABLE_1_VT_BUSY
                       );
}


/*------------------------------------------------------------------------------------------*\
 * Configure VLAN on the switch                                                             *
\*------------------------------------------------------------------------------------------*/
void ar_config_vlan_group(cpphy_mdio_t   *mdio,
                          unsigned short  vid,
                          unsigned short  portset,
                          unsigned short  tagged_portset __attribute__ ((unused)),
                          unsigned char   port,
                          unsigned char   FID __attribute__ ((unused)),
                          unsigned char   use_as_default __attribute__ ((unused))) {
    t_cpphy_switch_config *switch_config  = &mdio->switch_config;
    unsigned short vid_group;

    if(!(mdio->Mode & CPPHY_SWITCH_MODE_WRITE))
        return; /*--- CPMAC_ERR_NO_VLAN_POSSIBLE; ---*/

    vid_group = cpphy_ar_get_vid_group(mdio);

    /*--- switch_config->map_vid_to_port[vid] = port; ---*/
    switch_config->map_vid_to_group[vid] = vid_group;
    assert(vid_group < 4097);
    switch_config->vidgroup[vid_group].port_in = port;
    switch_config->vidgroup[vid_group].vid = vid;
    switch_config->vidgroup[vid_group].portset = portset;
    add_vlan_vid(mdio, vid, portset);
    switch_config->map_portset_out[portset] = vid;

    DEB_INFOTRC("[%s] Setting VID %#x with portmask %#x\n", __FUNCTION__, vid, portset);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned short cpphy_ar_get_vid_group(cpphy_mdio_t *mdio) {
    t_cpphy_switch_config *switch_config  = &mdio->switch_config;
    unsigned short vid_group;

    for(vid_group = 0; vid_group < switch_config->max_vid_groups; vid_group++) {
        if(!switch_config->vidgroup[vid_group].used) {
            break;
        }
    }
    if(vid_group >= switch_config->max_vid_groups) {
        DEB_ERR("[%s] Error! No free VLAN groups! Using 0 anyway!\n", __FUNCTION__);
        vid_group = 0;
    }
    switch_config->vidgroup[vid_group].used = 1;
    return vid_group;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned short cpphy_ar_get_new_vid(cpphy_mdio_t  *mdio,
                                    unsigned char  device __attribute__ ((unused))) {
    t_cpphy_switch_config *switch_config  = &mdio->switch_config;
    unsigned short vid;

    for(vid = 4; vid < 4096; vid++) {
        if(!cpphy_switch_check_assigned_vid(mdio, vid)) {
            switch_config->assigned_vids[switch_config->assigned_vids_count++] = vid;
            return vid;
        }
    }
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpphy_ar_set_default_vid(cpphy_mdio_t   *mdio, 
                              unsigned char   device, 
                              unsigned short  portset) {
    t_cpphy_switch_config *switch_config  = &mdio->switch_config;
    unsigned short vid = 0, port, first_port;

    first_port = 1;
    for(port = 0; port < AVM_CPMAC_MAX_PORTS; port++) {
        if((portset & (1 << port)) == 0) {
            continue; /* Port is not in portset */
        }
        switch_config->port[port].port_vlan_mask = portset;
        if(port == switch_config->cpu_port)  {
            continue; /* No tag for the CPU port needed */
        }
        if(first_port) {
            vid = cpphy_ar_get_new_vid(mdio, device);
            if(vid == 0) {
                DEB_ERR("[%s] Error! No unassigned VID (for default) available!\n", __FUNCTION__);
                return;
            }
            assert(mdio->f->config_vlan_group);
            DEB_INFOTRC("[%s] Setting default VID %#x for '%s' (portset %#x)\n",
                        __FUNCTION__, vid, mdio->switch_config.device[device].name, portset);
            mdio->f->config_vlan_group(mdio, vid, portset, 0, port, 0, first_port);
            switch_config->port[switch_config->cpu_port].vid = vid;
            switch_config->device[device].vid = vid;
        }
        switch_config->port[port].vid = vid;
        first_port = 0;
    }
    switch_config->port[switch_config->cpu_port].keep_tag_outgoing = 1;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar_config_ports(cpphy_mdio_t *mdio) {
    t_cpphy_switch_config *switch_config = &mdio->switch_config;
    unsigned int port, RegData;

    if(!(mdio->Mode & CPPHY_SWITCH_MODE_WRITE))
        return; /*--- CPMAC_ERR_NO_VLAN_POSSIBLE; ---*/

    assert(switch_config->ports < 7);

    for(port = 0; port < switch_config->ports; port++) {
        if(!(switch_config->used_portset & (1 << port)))
            continue;
        ar8216_mdio_write32(mdio, 
                            (port + 1) * 0x100 + AR8216_PORT_CONTROL_OFFSET,
                            ATH_PORT_CTRL_LEARN_EN
                            | ATH_PORT_CTRL_EG_VLAN_MODE((switch_config->port[port].keep_tag_outgoing) ? 0 : 1)
                            | ATH_PORT_CTRL_PORTSTATE_FORWARD
                            );
        /* Current port should not be in port VLAN mask for Atheros switches */
        /* Stay clear of changing reserved bits! */
        switch_config->port[port].port_vlan_mask &= ~(1 << port);
        RegData = ar8216_mdio_read32(mdio, (port + 1) * 0x100 + AR8216_PORT_VLAN_OFFSET);
        RegData &= ~(ATH_PORT_VLAN_VID_MASK | ATH_PORT_VLAN_VID_MEM_MASK | ATH_PORT_VLAN_8021_QMODE_MASK);
        RegData |=   ATH_PORT_VLAN_VID_VALUE(switch_config->port[port].vid)
                   | ATH_PORT_VLAN_VID_MEM_VALUE(switch_config->port[port].port_vlan_mask)
                   | ATH_PORT_VLAN_8021_QMODE_VALUE(ATH_PORT_VLAN_8021_QMODE_FALLBACK);
        ar8216_mdio_write32(mdio, (port + 1) * 0x100 + AR8216_PORT_VLAN_OFFSET, RegData);
        DEB_TRC("[%s] Set port %u: %s tagging, VID %#x (%u), portset %#x\n",
                __FUNCTION__,
                port,
                switch_config->port[port].keep_tag_outgoing ? "keep" : "remove",
                switch_config->port[port].vid,
                switch_config->port[port].vid,
                switch_config->port[port].port_vlan_mask);
        DEB_TRC("[%s] Port VLAN register port %u: %#x\n",
                __FUNCTION__,
                port,
                ar8216_mdio_read32(mdio, 
                                   (port + 1) * 0x100 + AR8216_PORT_VLAN_OFFSET));
    }
}


