/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : apload.h
// Description  : Information for AP load from Host
// ===========================================================================

#ifndef _APLOAD_H_
#define _APLOAD_H_

typedef enum
{
    SIM_OK = 0,         
    SIM_PMEM_ADDR,     
    SIM_DMEM_ADDR,    
    SIM_DMEM_ALIGN,  
    SIM_IO_NOPORT,  
    SIM_IO_BADPORT,
    SIM_DEC_BADINST,    
    SIM_STK_FULL,       
    SIM_STK_EMPTY,      
    SIM_AP_ID,          
    SIM_AP_EVENT_ERR,   
    SIM_ALLOC_ERR,      
    SIM_SLAVEMEM_ALLOC_ERR, 
    SIM_RAMQ_WRONG_PARAMS,  
    SIM_MAX_BM_CLUSTERS_TOO_SMALL, 
    SIM_PARAM_INITIALIZED_ALREADY, 
    SIM_EGRESS_TYPE,    
    SIM_QUEUE_NUM,      
    SIM_QUEUE_SIZE,     
    SIM_ALREADY_USED_QUEUE, 
    SIM_BAD_HASH_MASK,  
    SIM_WRONG_FUNCTION, 
    SIM_PARAM_NOT_INITIALIZED, 
    SIM_ALREADY_DISABLED_ENTRY,  
    SIM_ENTRY_COLLISION, 
    SIM_ENTRY_DOESNT_EXIST, 
    SIM_MATCHING_ENTRY, 
    SIM_WRONG_FEATURE,  
    SIM_MAC_LIST_FULL,  
    SIM_MAC_LIST_EMPTY,     
    SIM_WRONG_MAC_ADDR,     
    SIM_PRELIMINARY_CALL,   
    SIM_LATE_CALL,   
    SIM_AP_SUSPENSION_ACTION, 
    SIM_PPPOE_MATCHING_ENTRY,  
    SIM_UNCOMPATIBLE_OPERATIONS_ERR,       
    SIM_AP_INPUT_QUEUE_ALMOST_FULL, 
    SIM_WRONG_PARAMS_ERR,       
    SIM_ETHERTYPE_LIST_FULL,    
    SIM_INCORRECT_MTU_SIZE,     

    SIM_AHB_BADADDR = 0x1000,   
    SIM_AHB_ALIGN,      
    SIM_AHB_ACTIVE,     
    SIM_AHB_UNDERRUN,   
    SIM_AHB_OVERRUN,    
} simResult_t;

#define _codeSectionLength(pref)		pref ## _codeSectionLength
#define _dataSectionLength(pref)        pref ## _dataSectionLength
#define _sDataSectionLength(pref)       pref ## _sDataSectionLength
#define _codeSection(pref)		        pref ## _codeSection
#define _dataSection(pref)      		pref ## _dataSection
#define _sDataSection(pref)       		pref ## _sDataSection
#define _startData(pref)		        pref ## _startData
#define _endData(pref)			        pref ## _endData
#define _startSData(pref)		        pref ## _startSData
#define _endSData(pref)			        pref ## _endSData
#define _startBSS(pref)			        pref ## _startBSS
#define _endBSS(pref)		    	    pref ## _endBSS
#define _startSBSS(pref)		        pref ## _startSBSS
#define _endSBSS(pref)		        	pref ## _endSBSS


// Code and data information from assembled program for AP

#define extern_declaration_ap_vars( ap_prefix ) \
extern unsigned long    _codeSectionLength( ap_prefix );\
extern unsigned long    _dataSectionLength( ap_prefix );\
extern unsigned long    _sDataSectionLength( ap_prefix );\
extern unsigned char    _codeSection( ap_prefix )[];\
extern unsigned char    _dataSection( ap_prefix )[];\
extern unsigned char    _sDataSection( ap_prefix )[];\
extern unsigned long    _startData( ap_prefix );\
extern unsigned long    _endData( ap_prefix );\
extern unsigned long    _startSData( ap_prefix );\
extern unsigned long    _endSData( ap_prefix );\
extern unsigned long    _startBSS( ap_prefix );\
extern unsigned long    _endBSS( ap_prefix );\
extern unsigned long    _startSBSS( ap_prefix );\
extern unsigned long    _endSBSS( ap_prefix );



#define APUInit( apId, ap_prefix, pConfigStruct)    loadAPCode(  apId,\
                         _codeSectionLength(ap_prefix),\
                         _dataSectionLength(ap_prefix),\
                         _sDataSectionLength(ap_prefix),\
                         (unsigned long)_codeSection(ap_prefix),\
                         (unsigned long)_dataSection(ap_prefix),\
                         (unsigned long)_sDataSection(ap_prefix),\
                         _startData(ap_prefix),\
                         _endData(ap_prefix),\
                         _startSData(ap_prefix),\
                         _endSData(ap_prefix),\
                         _startBSS(ap_prefix),\
                         _endBSS(ap_prefix),\
                         _startSBSS(ap_prefix),\
                         _endSBSS(ap_prefix),\
                         pConfigStruct );


simResult_t loadAPCode( unsigned char  apId,
                        unsigned long codeSectionLength,
                        unsigned long _dataSectionLength,
                        unsigned long _sDataSectionLength,
                        unsigned long codeSection,
                        unsigned long _dataSection,
                        unsigned long _sDataSection,
                        unsigned long startData,
                        unsigned long _endData,
                        unsigned long startSData,
                        unsigned long _endSData,
                        unsigned long startBSS,
                        unsigned long _endBSS,
                        unsigned long startSBSS,
                        unsigned long _endSBSS,
                        void *pConfigStruct );
#endif /* _APLOAD_H_ */

